@@ -490,7 +490,7 @@ pgLsnFromStr(const String *lsn)
     {
         StringList *lsnPart = strLstNewSplit(lsn, FSLASH_STR);
 
-        CHECK(strLstSize(lsnPart) == 2);
+        CHECK(FormatError, strLstSize(lsnPart) == 2, "lsn requires two parts");
 
         result = (cvtZToUInt64Base(strZ(strLstGet(lsnPart, 0)), 16) << 32) + cvtZToUInt64Base(strZ(strLstGet(lsnPart, 1)), 16);
     }
@@ -540,6 +540,25 @@ pgLsnFromWalSegment(const String *walSegment, unsigned int walSegmentSize)
         (cvtZToUInt64Base(strZ(strSubN(walSegment, 16, 8)), 16) * walSegmentSize));
 }
 
+/**********************************************************************************************************************************/
+uint32_t
+pgTimelineFromWalSegment(const String *const walSegment)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(STRING, walSegment);
+    FUNCTION_TEST_END();
+
+    ASSERT(walSegment != NULL);
+    ASSERT(strSize(walSegment) == 24);
+
+    char buffer[9];
+
+    strncpy(buffer, strZ(walSegment), sizeof(buffer) - 1);
+    buffer[sizeof(buffer) - 1] = '\0';
+
+    FUNCTION_TEST_RETURN(cvtZToUIntBase(buffer, 16));
+}
+
 /**********************************************************************************************************************************/
 StringList *
 pgLsnRangeToWalSegmentList(
