@@ -4,9 +4,7 @@ S3 Storage File Write
 #include "build.auto.h"
 
 #include "common/debug.h"
-#include "common/io/write.intern.h"
 #include "common/log.h"
-#include "common/memContext.h"
 #include "common/type/object.h"
 #include "common/type/xml.h"
 #include "storage/s3/write.h"
@@ -33,7 +31,6 @@ Object type
 ***********************************************************************************************************************************/
 typedef struct StorageWriteS3
 {
-    MemContext *memContext;                                         // Object mem context
     StorageWriteInterface interface;                                // Interface
     StorageS3 *storage;                                             // Storage that created this object
 
@@ -68,7 +65,7 @@ storageWriteS3Open(THIS_VOID)
     ASSERT(this->partBuffer == NULL);
 
     // Allocate the part buffer
-    MEM_CONTEXT_BEGIN(this->memContext)
+    MEM_CONTEXT_BEGIN(THIS_MEM_CONTEXT())
     {
         this->partBuffer = bufNew(this->partSize);
     }
@@ -131,7 +128,7 @@ storageWriteS3PartAsync(StorageWriteS3 *this)
                             .query = httpQueryAdd(httpQueryNewP(), S3_QUERY_UPLOADS_STR, EMPTY_STR)))));
 
             // Store the upload id
-            MEM_CONTEXT_BEGIN(this->memContext)
+            MEM_CONTEXT_BEGIN(THIS_MEM_CONTEXT())
             {
                 this->uploadId = xmlNodeContent(xmlNodeChild(xmlRoot, S3_XML_TAG_UPLOAD_ID_STR, true));
                 this->uploadPartList = strLstNew();
@@ -144,7 +141,7 @@ storageWriteS3PartAsync(StorageWriteS3 *this)
         httpQueryAdd(query, S3_QUERY_UPLOAD_ID_STR, this->uploadId);
         httpQueryAdd(query, S3_QUERY_PART_NUMBER_STR, strNewFmt("%u", strLstSize(this->uploadPartList) + 1));
 
-        MEM_CONTEXT_BEGIN(this->memContext)
+        MEM_CONTEXT_BEGIN(THIS_MEM_CONTEXT())
         {
             this->request = storageS3RequestAsyncP(
                 this->storage, HTTP_VERB_PUT_STR, this->interface.name, .query = query, .content = this->partBuffer);
@@ -219,7 +216,7 @@ storageWriteS3Close(THIS_VOID)
             if (this->uploadId != NULL)
             {
                 // If there is anything left in the part buffer then write it
-                if (bufUsed(this->partBuffer) > 0)
+                if (!bufEmpty(this->partBuffer))
                     storageWriteS3PartAsync(this);
 
                 // Complete prior async request, if any
@@ -236,10 +233,18 @@ storageWriteS3Close(THIS_VOID)
                 }
 
                 // Finalize the multi-part upload
-                storageS3RequestP(
+                HttpRequest *request = storageS3RequestAsyncP(
                     this->storage, HTTP_VERB_POST_STR, this->interface.name,
                     .query = httpQueryAdd(httpQueryNewP(), S3_QUERY_UPLOAD_ID_STR, this->uploadId),
                     .content = xmlDocumentBuf(partList));
+                HttpResponse *response = storageS3ResponseP(request);
+
+                // Error if there is no etag in the result. This indicates that the request did not succeed despite the success code.
+                if (xmlNodeChild(
+                        xmlDocumentRoot(xmlDocumentNewBuf(httpResponseContent(response))), S3_XML_TAG_ETAG_STR, false) == NULL)
+                {
+                    httpRequestError(request, response);
+                }
             }
             // Else upload all the data in a single put
             else
@@ -268,19 +273,18 @@ storageWriteS3New(StorageS3 *storage, const String *name, size_t partSize)
 
     StorageWrite *this = NULL;
 
-    MEM_CONTEXT_NEW_BEGIN("StorageWriteS3")
+    OBJ_NEW_BEGIN(StorageWriteS3)
     {
-        StorageWriteS3 *driver = memNew(sizeof(StorageWriteS3));
+        StorageWriteS3 *driver = OBJ_NEW_ALLOC();
 
         *driver = (StorageWriteS3)
         {
-            .memContext = MEM_CONTEXT_NEW(),
             .storage = storage,
             .partSize = partSize,
 
             .interface = (StorageWriteInterface)
             {
-                .type = STORAGE_S3_TYPE_STR,
+                .type = STORAGE_S3_TYPE,
                 .name = strDup(name),
                 .atomic = true,
                 .createPath = true,
@@ -298,7 +302,7 @@ storageWriteS3New(StorageS3 *storage, const String *name, size_t partSize)
 
         this = storageWriteNew(driver, &driver->interface);
     }
-    MEM_CONTEXT_NEW_END();
+    OBJ_NEW_END();
 
     FUNCTION_LOG_RETURN(STORAGE_WRITE, this);
 }
